"use strict";
/**
 * Vanilla JS Match Height
 *
 * v1.0.6
 *
 * Josh Donnell
 *
 * @class
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var defaultOptions = {
    parent: null,
    byRow: true,
    timeOut: 50,
};
var MatchHeight = /** @class */ (function () {
    function MatchHeight(element, options) {
        if (options === void 0) { options = {}; }
        var _this = this;
        // Merge the default and the user options and set to the Options type
        this.options = __assign(__assign({}, defaultOptions), options);
        // Set the element
        this.element = element;
        // Check for class element
        if (this.element) {
            // Set heights on DOM load
            if (document.readyState !== 'loading') {
                this.init();
            }
            else {
                document.addEventListener('DOMContentLoaded', function () {
                    _this.init();
                });
            }
            // Update heights and rows on resize
            window.addEventListener('resize', function () {
                // Set a timeout to allow sliders and other JS to trigger first
                utilSetTimeout(function () {
                    _this.update();
                }, 100);
            });
        }
        else {
            // Sent the no class error to the console
            this.error('class');
        }
    }
    /**
     * Simple function to run on inital load
     * after the dom is registered
     */
    MatchHeight.prototype.init = function () {
        var _this = this;
        utilSetTimeout(function () {
            _this.reset();
        }, this.options.timeOut);
    };
    /**
     * Find all elements from parent
     * or from gloal document and
     * cache for updates.
     */
    MatchHeight.prototype.findElements = function () {
        var _this = this;
        // Set type
        var elementsInDom;
        // Check if parent and if parent has elements
        if (this.options.parent &&
            document.querySelectorAll(this.options.parent).length) {
            // Loop over to get the elements
            document
                .querySelectorAll(this.options.parent)
                .forEach(function (parentGroup) {
                // Find all elements in parent group
                elementsInDom = parentGroup.querySelectorAll(_this.element);
                // Create a group for each of the parents
                _this.createGroup(elementsInDom, parentGroup);
            });
        }
        else {
            // Find elements in the global document
            elementsInDom = document.querySelectorAll(this.element);
            // Create a group by
            this.createGroup(elementsInDom);
        }
        // find the rows based on the offset of the page
        this.findRows();
    };
    /**
     * Groups rows by offset if set
     * when registering match height
     */
    MatchHeight.prototype.findRows = function () {
        var _this = this;
        // Check for elements before continuing
        if (this.elements) {
            // Loop over each of the elements
            this.elements.map(function (group) {
                // Setup empty array to add sorted heights into
                var groupedHeights = [];
                //Loop over all single elements
                group.elements.forEach(function (element) {
                    var _a;
                    // Set a default offset to 0 to be used for grouping
                    var offset = 0;
                    // Check if ByRow is enabled, this is set to true by default
                    if (_this.options.byRow) {
                        // Calculate the elements offset to the page
                        offset = Math.round(element.getBoundingClientRect().top + window.scrollY);
                    }
                    // Gorup the elements by offset
                    groupedHeights[offset] = (_a = groupedHeights[offset]) !== null && _a !== void 0 ? _a : {
                        offset: offset,
                        elements: [],
                    };
                    // Push the elements to the correct offset
                    groupedHeights[offset].elements.push(element);
                });
                groupedHeights.map(function (row) {
                    group.rows.push({
                        height: 0,
                        offset: row.offset,
                        elements: row.elements,
                    });
                });
            });
            // Set the heights for the groups
            this.setHeights();
        }
    };
    /**
     * Create group from elements and a parent
     * also set the data back to the elements array
     *
     * @param {Array} elements
     * @param {Element} parent
     */
    MatchHeight.prototype.createGroup = function (elements, parent) {
        // Put to the cache to be used for updates
        var group = {
            elements: elements,
            rows: [],
            parent: parent,
        };
        this.elements.push(group);
    };
    /**
     * Set the largest heights in our
     * groups of elements.
     *
     */
    MatchHeight.prototype.setHeights = function () {
        //Check to make sure we have an elements
        if (this.elements) {
            // Loop over each of the elements instances
            for (var _i = 0, _a = this.elements; _i < _a.length; _i++) {
                var group = _a[_i];
                // Add the height for elements in the row
                group.rows.map(function (row) {
                    // Set a temp array for the heights
                    var heights = [];
                    row.elements.forEach(function (element) {
                        // Remove any set heights to calcualte the raw height
                        element.style.height = null;
                        // Get the height and push into the heights array
                        heights.push(element.offsetHeight);
                    });
                    // Set the heights to the max height
                    row.height = Math.max.apply(Math, heights);
                    // Apply the max height
                    row.elements.forEach(function (element) {
                        element.style.height = "".concat(row.height, "px");
                    });
                });
            }
        }
        else {
            this.error();
        }
    };
    /**
     * Function used to trigger updades,
     * these can be triggered manually or
     *  are already set to run on resize
     */
    MatchHeight.prototype.update = function () {
        // Check for elements and trigger heights update
        if (this.elements.length) {
            // Reset the rows
            this.findRows();
        }
        else {
            // Log an error if no elements can be found in the cache.
            this.error();
        }
    };
    /**
     * Reset all heights and refind groups and rows
     * for all the elements in the group.
     * This is best to be used when updating the
     * content of a page as the prevoius values
     * will be cached in out elements object.
     */
    MatchHeight.prototype.reset = function () {
        // Bust the elements cache back to empty
        this.elements = [];
        // Find the elements from the assigned class and parent
        // and refind rows and set the heights again.
        this.findElements();
    };
    /**
     * Basic function to log the current cache
     * for the class. Can be useful if you are facing an issue.
     */
    MatchHeight.prototype.debug = function () {
        var _this = this;
        // Check to make sure we have the cached elements
        utilSetTimeout(function () {
            if (_this.elements) {
                // Loop over elements
                _this.elements.forEach(function (group, index) {
                    // Tell the console there is a new group
                    if (group.parent) {
                        console.log("Element: ".concat(_this.element, " Parent: ").concat(group.parent.tagName, ".").concat(group.parent.className));
                    }
                    else {
                        console.log("Element: ".concat(_this.element));
                    }
                    group.rows.forEach(function (row, index) {
                        // Log a list of the cached elements
                        console.log("Offset Group: ".concat(row.offset));
                        console.log(row);
                    });
                    console.log('\n\n');
                });
            }
            else {
                // Log an error if no cached elements can be found.
                console.log("\u26D4 No cached MatchHeight elements found for \"".concat(_this.element, "\" \n\n Make sure this element exists on the current page."));
            }
        }, this.options.timeOut + 250);
    };
    /**
     * Basic function to log the current cache
     * for the class. Can be useful if you are facing an issue.
     *
     * @param {string} type
     */
    MatchHeight.prototype.error = function (type) {
        // Log Error if no class found
        if (type === void 0) { type = ''; }
        if (type == 'class') {
            console.error('⛔ Missing class name in new MatchHeight()');
        }
        else {
            console.error("\u26D4 Can't trigger update as no elements found for the MatchHeight element ".concat(this.element));
        }
    };
    return MatchHeight;
}());
var utilSetTimeout = function (callback, delay) {
    var startTime = performance.now();
    var handleCallback = function (currentTime) {
        var elapsedTime = currentTime - startTime;
        if (elapsedTime >= delay) {
            callback();
        }
        else {
            requestAnimationFrame(handleCallback);
        }
    };
    requestAnimationFrame(handleCallback);
};
// Export to use our MatchHeight class
exports.default = MatchHeight;
